from com import Configurator, msgs
from theme import theme

import os


_PATH = os.path.dirname(__file__)

class Tour(Configurator):

    ICON = theme.mb_get_started
    TITLE = "Get Started"
    DESCRIPTION = "Learn how to use MediaBox"
    

    def __init__(self):
    
        self.__current_page = 0
        self.__pages = []
    
        data = open(os.path.join(_PATH, "tour_C.dat")).read()
        self.__parse_tour(data)
    
        Configurator.__init__(self)
        self.connect_button_released(self.__on_click)
        
        
    def __on_click(self, px, py):
    
        w, h = self.get_size()
        if (px < w / 2):
            self.__go_back()
        else:
            self.__go_forward()
            
            
    def __go_back(self):
    
        if (self.__current_page > 0):
            self.__current_page -= 1
            self.render()

    def __go_forward(self):
    
        if (self.__current_page < len(self.__pages) - 1):
            self.__current_page += 1
            self.render()
    
        
    def __parse_tour(self, data):
    
        self.__pages = []
        title = ""
        image = ""
        text = ""
        
        for line in data.splitlines():
            if (line.startswith("BEGIN")):
                title = line[6:].strip()
                image = ""
                text = ""
            elif (line.startswith("IMAGE ")):
                image = line[6:].strip()
            elif (line.startswith("END")):
                self.__pages.append((title, image, text))
            else:
                text += line + "\n"
        #end for
        
        
    def render_this(self):
    
        x, y = self.get_screen_pos()
        w, h = self.get_size()
        screen = self.get_screen()
        
        title, image, text = self.__pages[self.__current_page]
        
        screen.fill_area(0, 0, w, h, theme.color_mb_background)
        
        screen.draw_text("%d/%d" % (self.__current_page + 1, len(self.__pages)),
                         theme.font_mb_tiny, w - 36, 4, "#aaaaaa")
        
        xpos = 4
        ypos = 4
        if (title):
            screen.draw_text(title, theme.font_mb_headline, xpos, ypos, "#ffffff")
            ypos += 60

        if (image):
            pbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(_PATH, image))
            screen.draw_pixbuf(pbuf, xpos, ypos)
            xpos += 200

        if (text):
            screen.draw_text(text, theme.font_mb_plain, xpos, ypos, "#ffffff")

